#include "ColorLCDShield.h"
#include "Histry.h"

void countInterruptFunc();
unsigned long getGeigerCounter();
int CountToSv100( long Count, long Coefficient );
char* ItoA5( unsigned int a );
char* microSvString( int count, int Coefficient );////1000が標準

#define PI 3.14

//LCD
LCDShield lcd;
//#define BACKGROUND  GREEN  //OK
//#define BACKGROUND  BLUE  //NG
#define BACKGROUND  RED  //NG
//#define BACKGROUND  BLACK  //OK
//#define BACKGROUND  WHITE  //OK

//ガイガーデータ
#define INTERVAL_MILLS 500  // milli sec
#define MEASURE_SEC 60      // 60秒
#define HISTRY_MAX 120      // 60sec/500msec
HISTRY_DATA_TYPE mHistryBuf[ HISTRY_MAX ] ;
Histry  mHistry ;

//ガイガーカウンタ(割り込みで incriment)
volatile unsigned long mGeigerCounter ;

/**************************
セットアップ
 *************************/
void setup()
{
    /* Initialize the LCD, set the contrast, clear the screen */
    //lcd.init(EPSON);  // This should only be init(EPSON) or init(PHILLIPS)
    lcd.init(PHILLIPS);
    lcd.contrast(40);
    lcd.clear(BACKGROUND);

    // ガイガー入力 IO割り込み http://www.arduino.cc/en/Reference/AttachInterrupt
    mGeigerCounter = 0 ;
    int CounterPin = 2 ;
    int CounterPinInterruptNo = 0 ;
    pinMode( CounterPin, INPUT ) ;
    digitalWrite( CounterPin, HIGH ) ;//turn on pullup resistors
    attachInterrupt( CounterPinInterruptNo, countInterruptFunc, FALLING ) ;//LOW, CHANGE, RISING, FALLING
}
#define DISP_SV_X 0
#define DISP_SV_Y 10
#define DISP_CM_X 0
#define DISP_CM_Y (DISP_SV_Y+18)
#define DISP_CA_X 0
#define DISP_CA_Y (DISP_CM_Y+18)
/**************************
ループ
 *************************/
void loop()
{
    char str[ 30 ] ;
    mHistry.setBuffer( HISTRY_MAX, mHistryBuf ) ;

    unsigned long lastMills = millis() ;
    unsigned long lastCount = getGeigerCounter() ;
    while( true ) {
        delay(100);
        if( ( millis() - lastMills ) <= INTERVAL_MILLS ) {
            //前回の処理から、INTERVAL_MILLS 以下の時間しか経ってない場合には何もしない。
            continue ;
        }
        //データの読み取りと、histry へのデータ追加
        lastMills += INTERVAL_MILLS ;
        unsigned long count = getGeigerCounter() ;
        unsigned long diff = count - lastCount ;
        lastCount = count ;
        mHistry.entryData( diff ) ;

        //カウント数の表示
        sprintf( str, "Count %ld ", mHistry.getSum() ) ;
        lcd.setStr(str, DISP_CM_Y, DISP_CM_X , BLUE, BACKGROUND);

        sprintf( str, "Total %ld ", getGeigerCounter() ) ;
        lcd.setStr(str, DISP_CA_Y, DISP_CA_X , BLUE, BACKGROUND);

        if( mHistry.howMany() < HISTRY_MAX ) {
            //しばらくお待ちください
            sprintf( str, "Wait %d   ", ( HISTRY_MAX - mHistry.howMany() ) / 2 );
            lcd.setStr(str, DISP_SV_Y, DISP_SV_X , BLUE, BACKGROUND);
            continue ;
        }
        //データを表示
        lcd.setStr( microSvString( mHistry.getSum(), 1000 ), DISP_SV_Y, DISP_SV_X , BLUE, BACKGROUND);
    }
}
/**************************
ガイガー割り込み
 *************************/
void countInterruptFunc()
{
    mGeigerCounter ++ ;
}
/**************************
ガイガーカウンタ読み込み
 *************************/
unsigned long getGeigerCounter()
{
    cli();                // disable global interrupts
    unsigned long tmp = mGeigerCounter ;
    sei();//割り込み許可
    return tmp ;
}

/******************************************************************************
機能要約 uSv/h の文字列を作り出す。
機能
引き数
戻り値
注意 浮動小数点は使いたくないのでこうなった。Coefficientは、標準で 1000
******************************************************************************/
char* microSvString( int count, int coefficient )
{
    static char buf[ 14 ] ;
    count = CountToSv100( count, coefficient ) ;
    char* str = ItoA5( count ) ;
    buf[ 0 ] = str[ 1 ] ;
    buf[ 1 ] = str[ 2 ] ;
    buf[ 2 ] = '.' ;
    buf[ 3 ] = str[ 3 ] ;
    buf[ 4 ] = str[ 4 ] ;
    strcpy( &buf[ 5 ], " uSv/h" ) ;
    return buf ;
}
/******************************************************************************
機能要約 シーベルトの Coefficient/10 倍の値を求める
機能
引き数
戻り値
注意 浮動小数点は使いたくないのでこうなった。Coefficientは、標準で 1000
******************************************************************************/
int CountToSv100( long Count, long Coefficient )
{
    long Answer ;
    //tprintf( "CountToSv100 count of histry = %ld\n", Count ) ;
    Answer = ( Count * Coefficient * 360 ) / MEASURE_SEC ;// 360は、経験的な値
    Answer /= 10000 ;//浮動小数点を使わない方が良いので、Coefficient倍して 1/10000倍している.
    //tprintf( "CountToSv100 Answer = %ld\n", Answer ) ;
    return ( int )Answer ;
}
/******************************************************************************
機能要約  int を5桁の10進文字列にする
機能
引数
  a 文字列にしたいデータ
戻り値
  文字列へのポインタ
注意
******************************************************************************/
char* ItoA5( unsigned int a )
{
	static char Buffer[ 6 ] ;
	static const unsigned int Div[] = { 10000U, 1000U, 100U, 10U, 1U } ;
	int i ;

	for( i = 0 ; i < 5 ; i ++ ) {
		Buffer[ i ] = ( a / Div[ i ] ) + '0' ;
		a %= Div[ i ] ;
	}
	Buffer[ i ] = 0 ;
	return( Buffer ) ;
}
