#include "Thread.h"
#define STACK_SIZE  (200)   //各タスク固定のスタックサイズ(short word数)
#define TASK_MAX    (3)     //タスク数(固定)

static void init_stacks( int lev );
static void dispatch(void);


typedef struct procs_ctl {  /* process context */
    void (*p_entry)( void );    /* process entry point */
    jmp_buf jbuf;
    unsigned short resv[16];    //安全のために確保しておく
#ifdef NO_TASK_STACK_CHECK
    char *stackcheck;
#endif
} PC;


//変数
static PC process_table[ TASK_MAX ];
static volatile PC *cpc; //カレントプロセスのコンテキスト

static void *stack_area_front = NULL;
static jmp_buf starter_jbuf;
static jmp_buf dispatch_jbuf;

static const char StackCheckString[] = "*STACK*" ;
//---------------------------------------------------------------------------
// マルチタスク処理の開始
//---------------------------------------------------------------------------
void startThread()
{
    if( setjmp( starter_jbuf )==0 ){
        init_stacks( TASK_MAX );
    }
    dispatch();
}
//---------------------------------------------------------------------------
// 全タスクのスタックを再帰的に初期化する。
// 各スタックは、呼び出した環境のスタックフレーム内部に入れ子状に確保される。 
//---------------------------------------------------------------------------
static void init_stacks( int lev )
{
    volatile unsigned short reserve[ STACK_SIZE ];
    reserve[0] = lev;   /* local variable 'volatile int level' の代替。 
                           LSI-C 86 ではローカル変数の配置が意図通りにならなかった */
    stack_area_front = (void *)reserve;  /* information for debug */
#ifdef NO_TASK_STACK_CHECK
    strcpy( ( char* )&reserve[50], StackCheckString );
    process_table[reserve[0]-1].stackcheck = ( char* )&reserve[50];
#endif
    if( reserve[0] ){  /* 以降では lev のかわりに reserve[0] を使用する */
        /* process 実行開始環境の設定 */
        /* N,N-1,N-2,,,0 */
        if( setjmp( process_table[reserve[0]-1].jbuf )==0 ){
            /* 残りのprosessについて処理...*/
            init_stacks( reserve[0]-1 );

        }else{  /* process 実行開始 */
            //entry pointから実行開始(sleep()などがdispatch()を呼ぶまで実行)
            //最初のプロセスがここを通過するまでは、cpc が確定していない。
            (*(cpc->p_entry))();

            //process の実行終了(process top level の return で、ここに戻る)
            longjmp( dispatch_jbuf, 0);
        }
    }
    longjmp( starter_jbuf,0 );
}
//---------------------------------------------------------------------------
//イベント管理の例として、スリープ処理。
// タイマカウント処理は、別途記述が必要。
//---------------------------------------------------------------------------
void sleep(unsigned long ticks)
{
    long i ;
    unsigned long lastMillis = millis() ;
    while(true){
        if( ( millis() - lastMillis ) > ticks ) {
            break;
        }
        if( setjmp( (_jmp_buf*)(cpc->jbuf) )==0 ){
            longjmp( (_jmp_buf*)dispatch_jbuf, 0);
        }
    }
}
//---------------------------------------------------------------------------
static void dispatch(void)
{
    volatile int i;
    while( 1 ){
        for( i=0; i<TASK_MAX; i++ ){
            cpc = &process_table[i];
#ifdef NO_TASK_STACK_CHECK
            if( strcmp(cpc->stackcheck, StackCheckString) ) {
                printf( " stack vioration "); 
            }
#endif
            if( setjmp( (_jmp_buf*)dispatch_jbuf )==0 ){//
                longjmp( (_jmp_buf*)(cpc->jbuf), 0 );
            }
        }
    }
}
static void dummyFunc0()
{
    while(true){
        Serial.print("0");
        sleep( 1000 ) ;
    }
}
void initThread()
{
    for( int i = 0 ; i < TASK_MAX ; i ++ ) {
        entryThread( dummyFunc0, i ) ;
    }
}
void entryThread( FUNC_THREAD func, int taskNo )
{
    process_table[taskNo].p_entry = func ;
}
